@@ -12,13 +12,14 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
 import collections
-import json
 import base64
 from contextlib import closing
 
 import eventlet
 from oslo_config import cfg
+from oslo_serialization import jsonutils
 import mock
 from nova import context
 from nova import exception
@@ -412,6 +413,41 @@ class LXDDriverTest(test.NoDBTestCase):
         lxd_driver.cleanup.assert_called_once_with(
             ctx, instance, network_info, block_device_info)
 
+    @mock.patch('nova.virt.configdrive.required_by', return_value=False)
+    def test_spawn_container_cleanup_fail(self, configdrive):
+        """Cleanup is called but also fail when container creation fails."""
+        self.client.containers.get.side_effect = (
+            lxdcore_exceptions.LXDAPIException(MockResponse(404)))
+        container = mock.Mock()
+        self.client.containers.create.return_value = container
+
+        ctx = context.get_admin_context()
+        instance = fake_instance.fake_instance_obj(
+            ctx, name='test', memory_mb=0)
+        image_meta = mock.Mock()
+        injected_files = mock.Mock()
+        admin_password = mock.Mock()
+        allocations = mock.Mock()
+        network_info = [_VIF]
+        block_device_info = mock.Mock()
+        virtapi = manager.ComputeVirtAPI(mock.MagicMock())
+
+        lxd_driver = driver.LXDDriver(virtapi)
+        lxd_driver.init_host(None)
+
+        container.start.side_effect = (
+            lxdcore_exceptions.LXDAPIException(MockResponse(500)))
+        lxd_driver.cleanup = mock.Mock()
+        lxd_driver.cleanup.side_effect = Exception("a bad thing")
+
+        self.assertRaises(
+            lxdcore_exceptions.LXDAPIException,
+            lxd_driver.spawn,
+            ctx, instance, image_meta, injected_files, admin_password,
+            allocations, network_info, block_device_info)
+        lxd_driver.cleanup.assert_called_once_with(
+            ctx, instance, network_info, block_device_info)
+
     def test_spawn_container_start_fail(self, neutron_failure=None):
         def container_get(*args, **kwargs):
             raise lxdcore_exceptions.LXDAPIException(MockResponse(404))
@@ -524,7 +560,8 @@ class LXDDriverTest(test.NoDBTestCase):
         self._test_spawn_instance_with_network_events(
             neutron_failure='timeout')
 
-    def test_destroy(self):
+    @mock.patch('nova.virt.lxd.driver.lockutils.lock')
+    def test_destroy(self, lock):
         mock_container = mock.Mock()
         mock_container.status = 'Running'
         self.client.containers.get.return_value = mock_container
@@ -545,7 +582,8 @@ class LXDDriverTest(test.NoDBTestCase):
         mock_container.stop.assert_called_once_with(wait=True)
         mock_container.delete.assert_called_once_with(wait=True)
 
-    def test_destroy_when_in_rescue(self):
+    @mock.patch('nova.virt.lxd.driver.lockutils.lock')
+    def test_destroy_when_in_rescue(self, lock):
         mock_stopped_container = mock.Mock()
         mock_stopped_container.status = 'Stopped'
         mock_rescued_container = mock.Mock()
@@ -579,7 +617,8 @@ class LXDDriverTest(test.NoDBTestCase):
         mock_rescued_container.stop.assert_called_once_with(wait=True)
         mock_rescued_container.delete.assert_called_once_with(wait=True)
 
-    def test_destroy_without_instance(self):
+    @mock.patch('nova.virt.lxd.driver.lockutils.lock')
+    def test_destroy_without_instance(self, lock):
         def side_effect(*args, **kwargs):
             raise lxdcore_exceptions.LXDAPIException(MockResponse(404))
         self.client.containers.get.side_effect = side_effect
@@ -796,6 +835,28 @@ class LXDDriverTest(test.NoDBTestCase):
         self.assertEqual(['root'], sorted(profile.devices.keys()))
         profile.save.assert_called_once_with(wait=True)
 
+    def test_detach_interface_not_found(self):
+        self.client.profiles.get.side_effect = lxdcore_exceptions.NotFound(
+            "404")
+
+        ctx = context.get_admin_context()
+        instance = fake_instance.fake_instance_obj(
+            ctx, name='test', memory_mb=0)
+        vif = {
+            'id': '0123456789abcdef',
+            'type': network_model.VIF_TYPE_OVS,
+            'address': '00:11:22:33:44:55',
+            'network': {
+                'bridge': 'fakebr'}}
+
+        lxd_driver = driver.LXDDriver(None)
+        lxd_driver.init_host(None)
+
+        lxd_driver.detach_interface(ctx, instance, vif)
+
+        self.vif_driver.unplug.assert_called_once_with(
+            instance, vif)
+
     def test_migrate_disk_and_power_off(self):
         container = mock.Mock()
         self.client.containers.get.return_value = container
@@ -1172,7 +1233,7 @@ class LXDDriverTest(test.NoDBTestCase):
         lxd_driver.client.host_info = lxd_config
         value = lxd_driver.get_available_resource(None)
         # This is funky, but json strings make for fragile tests.
-        value['cpu_info'] = json.loads(value['cpu_info'])
+        value['cpu_info'] = jsonutils.loads(value['cpu_info'])
 
         self.assertEqual(expected, value)
 
@@ -1269,7 +1330,7 @@ class LXDDriverTest(test.NoDBTestCase):
         lxd_driver.client.host_info = lxd_config
         value = lxd_driver.get_available_resource(None)
         # This is funky, but json strings make for fragile tests.
-        value['cpu_info'] = json.loads(value['cpu_info'])
+        value['cpu_info'] = jsonutils.loads(value['cpu_info'])
 
         self.assertEqual(expected, value)
 
