@@ -17,7 +17,6 @@ from __future__ import absolute_import
 
 import errno
 import io
-import json
 import os
 import platform
 import pwd
@@ -352,7 +351,7 @@ def _sync_glance_image_to_lxd(client, context, image_ref):
                         'hw_architecture',
                         obj_fields.Architecture.from_host()),
                     'creation_date': int(os.stat(image_file).st_ctime)}
-                metadata_yaml = json.dumps(
+                metadata_yaml = jsonutils.dumps(
                     metadata, sort_keys=True, indent=4,
                     separators=(',', ': '),
                     ensure_ascii=False).encode('utf-8') + b"\n"
@@ -612,10 +611,14 @@ class LXDDriver(driver.ComputeDriver):
 
             self.firewall_driver.apply_instance_filter(
                 instance, network_info)
-        except lxd_exceptions.LXDAPIException as e:
+        except lxd_exceptions.LXDAPIException:
             with excutils.save_and_reraise_exception():
-                self.cleanup(
-                    context, instance, network_info, block_device_info)
+                try:
+                    self.cleanup(
+                        context, instance, network_info, block_device_info)
+                except Exception as e:
+                    LOG.warn('The cleanup process failed with: %s. This '
+                             'error may or not may be relevant', e)
 
     def destroy(self, context, instance, network_info, block_device_info=None,
                 destroy_disks=True, migrate_data=None):
@@ -627,27 +630,34 @@ class LXDDriver(driver.ComputeDriver):
         See `nova.virt.driver.ComputeDriver.destroy` for more
         information.
         """
-        try:
-            container = self.client.containers.get(instance.name)
-            if container.status != 'Stopped':
-                container.stop(wait=True)
-            container.delete(wait=True)
-            if (instance.vm_state == vm_states.RESCUED):
-                rescued_container = self.client.containers.get(
-                    '%s-rescue' % instance.name)
-                if rescued_container.status != 'Stopped':
-                    rescued_container.stop(wait=True)
-                rescued_container.delete(wait=True)
-        except lxd_exceptions.LXDAPIException as e:
-            if e.response.status_code == 404:
-                LOG.warning("Failed to delete instance. "
-                            "Container does not exist for {instance}."
-                            .format(instance=instance.name))
-            else:
-                raise
-        finally:
-            self.cleanup(
-                context, instance, network_info, block_device_info)
+        lock_path = os.path.join(CONF.instances_path, 'locks')
+
+        with lockutils.lock(
+                lock_path, external=True,
+                lock_file_prefix='lxd-container-{}'.format(instance.name)):
+            # TODO(sahid): Each time we get a container we should
+            # protect it by using a mutex.
+            try:
+                container = self.client.containers.get(instance.name)
+                if container.status != 'Stopped':
+                    container.stop(wait=True)
+                container.delete(wait=True)
+                if (instance.vm_state == vm_states.RESCUED):
+                    rescued_container = self.client.containers.get(
+                        '{}-rescue'.format(instance.name))
+                    if rescued_container.status != 'Stopped':
+                        rescued_container.stop(wait=True)
+                    rescued_container.delete(wait=True)
+            except lxd_exceptions.LXDAPIException as e:
+                if e.response.status_code == 404:
+                    LOG.warning("Failed to delete instance. "
+                                "Container does not exist for {instance}."
+                                .format(instance=instance.name))
+                else:
+                    raise
+            finally:
+                self.cleanup(
+                    context, instance, network_info, block_device_info)
 
     def cleanup(self, context, instance, network_info, block_device_info=None,
                 destroy_disks=True, migrate_data=None, destroy_vifs=True):
@@ -800,25 +810,34 @@ class LXDDriver(driver.ComputeDriver):
         profile.save(wait=True)
 
     def detach_interface(self, context, instance, vif):
-        profile = self.client.profiles.get(instance.name)
-        devname = lxd_vif.get_vif_devname(vif)
+        try:
+            profile = self.client.profiles.get(instance.name)
+            devname = lxd_vif.get_vif_devname(vif)
 
-        # NOTE(jamespage): Attempt to remove device using
-        #                  new style tap naming
-        if devname in profile.devices:
-            del profile.devices[devname]
-            profile.save(wait=True)
-        else:
-            # NOTE(jamespage): For upgrades, scan devices
-            #                  and attempt to identify
-            #                  using mac address as the
-            #                  device will *not* have a
-            #                  consistent name
-            for key, val in profile.devices.items():
-                if val.get('hwaddr') == vif['address']:
-                    del profile.devices[key]
-                    profile.save(wait=True)
-                    break
+            # NOTE(jamespage): Attempt to remove device using
+            #                  new style tap naming
+            if devname in profile.devices:
+                del profile.devices[devname]
+                profile.save(wait=True)
+            else:
+                # NOTE(jamespage): For upgrades, scan devices
+                #                  and attempt to identify
+                #                  using mac address as the
+                #                  device will *not* have a
+                #                  consistent name
+                for key, val in profile.devices.items():
+                    if val.get('hwaddr') == vif['address']:
+                        del profile.devices[key]
+                        profile.save(wait=True)
+                        break
+        except lxd_exceptions.NotFound:
+            # This method is called when an instance get destroyed. It
+            # could happen that Nova to receive an event
+            # "vif-delete-event" after the instance is destroyed which
+            # result the lxd profile not exist.
+            LOG.debug("lxd profile for instance {instance} does not exist. "
+                      "The instance probably got destroyed before this method "
+                      "got called.".format(instance=instance.name))
 
         self.vif_driver.unplug(instance, vif)
 
@@ -841,7 +860,7 @@ class LXDDriver(driver.ComputeDriver):
 
         with lockutils.lock(
                 lock_path, external=True,
-                lock_file_prefix=('lxd-snapshot-%s' % instance.name)):
+                lock_file_prefix='lxd-container-{}'.format(instance.name)):
 
             update_task_state(task_state=task_states.IMAGE_PENDING_UPLOAD)
 
@@ -1257,9 +1276,10 @@ class LXDDriver(driver.ComputeDriver):
             "volatile.tap5fd6808a-7b.name": "eth0"
         }
         """
-        container_id_map = json.loads(
+        container_id_map = jsonutils.loads(
             container.config['volatile.last_state.idmap'])
-        uid_map = filter(lambda id_map: id_map.get("Isuid"), container_id_map)
+        uid_map = list(filter(lambda id_map: id_map.get("Isuid"),
+                              container_id_map))
         if uid_map:
             storage_id = uid_map[0].get("Hostid", 0)
         else:
