@@ -20,10 +20,9 @@ from nova import exception
 from nova import utils
 from nova.network import model as network_model
 from nova.network import os_vif_util
-from nova.network import linux_utils as network_utils
+from nova.privsep import linux_net
 
 import os_vif
-from vif_plug_ovs import linux_net
 
 
 CONF = conf.CONF
@@ -48,14 +47,14 @@ def _create_veth_pair(dev1_name, dev2_name, mtu=None):
     deleting any previous devices with those names.
     """
     for dev in [dev1_name, dev2_name]:
-        network_utils.delete_net_dev(dev)
+        linux_net.delete_net_dev(dev)
 
     utils.execute('ip', 'link', 'add', dev1_name, 'type', 'veth', 'peer',
                   'name', dev2_name, run_as_root=True)
 
     for dev in [dev1_name, dev2_name]:
         utils.execute('ip', 'link', 'set', dev, 'up', run_as_root=True)
-        network_utils.set_device_mtu(dev, mtu)
+        linux_net.set_device_mtu(dev, mtu)
 
 
 def _add_bridge_port(bridge, dev):
@@ -91,6 +90,44 @@ def _get_tap_config(vif):
     return {'mac_address': vif['address']}
 
 
+def _ovs_vsctl(args):
+    full_args = ['ovs-vsctl', '--timeout=%s' % CONF.ovs_vsctl_timeout] + args
+    try:
+        return utils.execute(*full_args, run_as_root=True)
+    except Exception as e:
+        LOG.error("Unable to execute %(cmd)s. Exception: %(exception)s",
+                  {'cmd': full_args, 'exception': e})
+        raise exception.OvsConfigurationFailure(inner_exception=e)
+
+
+def _create_ovs_vif_cmd(bridge, dev, iface_id, mac,
+                        instance_id, interface_type=None):
+    cmd = ['--', '--if-exists', 'del-port', dev, '--',
+           'add-port', bridge, dev,
+           '--', 'set', 'Interface', dev,
+           'external-ids:iface-id=%s' % iface_id,
+           'external-ids:iface-status=active',
+           'external-ids:attached-mac=%s' % mac,
+           'external-ids:vm-uuid=%s' % instance_id]
+    if interface_type:
+        cmd += ['type=%s' % interface_type]
+    return cmd
+
+
+def _create_ovs_vif_port(bridge, dev, iface_id, mac, instance_id,
+                         mtu=None, interface_type=None):
+    _ovs_vsctl(_create_ovs_vif_cmd(bridge, dev, iface_id,
+                                   mac, instance_id,
+                                   interface_type))
+    linux_net.set_device_mtu(dev, mtu)
+
+
+def _delete_ovs_vif_port(bridge, dev, delete_dev=True):
+    _ovs_vsctl(['--', '--if-exists', 'del-port', bridge, dev])
+    if delete_dev:
+        linux_net.delete_net_dev(dev)
+
+
 CONFIG_GENERATORS = {
     'bridge': _get_bridge_config,
     'ovs': _get_ovs_config,
@@ -125,21 +162,21 @@ def _post_plug_wiring_veth_and_bridge(instance, vif):
     mtu = network.get_meta('mtu') if network else None
     v1_name = get_vif_devname(vif)
     v2_name = get_vif_internal_devname(vif)
-    if not network_utils.device_exists(v1_name):
+    if not linux_net.device_exists(v1_name):
         _create_veth_pair(v1_name, v2_name, mtu)
         if _is_ovs_vif_port(vif):
             # NOTE(jamespage): wire tap device directly to ovs bridge
-            linux_net.create_ovs_vif_port(vif['network']['bridge'],
-                                          v1_name,
-                                          vif['id'],
-                                          vif['address'],
-                                          instance.uuid,
-                                          mtu)
+            _create_ovs_vif_port(vif['network']['bridge'],
+                                 v1_name,
+                                 vif['id'],
+                                 vif['address'],
+                                 instance.uuid,
+                                 mtu)
         else:
             # NOTE(jamespage): wire tap device linux bridge
             _add_bridge_port(config['bridge'], v1_name)
     else:
-        network_utils.set_device_mtu(v1_name, mtu)
+        linux_net.set_device_mtu(v1_name, mtu)
 
 
 POST_PLUG_WIRING = {
@@ -189,10 +226,10 @@ def _post_unplug_wiring_delete_veth(instance, vif):
     v1_name = get_vif_devname(vif)
     try:
         if _is_ovs_vif_port(vif):
-            linux_net.delete_ovs_vif_port(vif['network']['bridge'],
-                                          v1_name, True)
+            _delete_ovs_vif_port(vif['network']['bridge'],
+                                 v1_name, True)
         else:
-            network_utils.delete_net_dev(v1_name)
+            linux_net.delete_net_dev(v1_name)
     except processutils.ProcessExecutionError:
         LOG.exception("Failed to delete veth for vif {}".foramt(vif),
                       instance=instance)
@@ -283,16 +320,16 @@ class LXDGenericVifDriver(object):
         # NOTE(jamespage): For nova-lxd this is really a veth pair
         #                  so that a) security rules get applied on the host
         #                  and b) that the container can still be wired.
-        if not network_utils.device_exists(v1_name):
+        if not linux_net.device_exists(v1_name):
             _create_veth_pair(v1_name, v2_name, mtu)
         else:
-            network_utils.set_device_mtu(v1_name, mtu)
+            linux_net.set_device_mtu(v1_name, mtu)
 
     def unplug_tap(self, instance, vif):
         """Unplug a VIF_TYPE_TAP virtual interface."""
         dev = get_vif_devname(vif)
         try:
-            network_utils.delete_net_dev(dev)
+            linux_net.delete_net_dev(dev)
         except processutils.ProcessExecutionError:
             LOG.exception("Failed while unplugging vif for instance",
                           instance=instance)
